#include <windows.h>
#include <winver.h>

VS_VERSION_INFO VERSIONINFO
FILEVERSION     2,0,0,1
PRODUCTVERSION  2,0,0,1
FILEFLAGSMASK   VS_FFI_FILEFLAGSMASK
FILEFLAGS       0
FILEOS          VOS__WINDOWS32
FILETYPE        VFT_DLL
FILESUBTYPE     VFT2_UNKNOWN

BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904B0"   /* LANG_ENGLISH/SUBLANG_ENGLISH_US, Unicode CP */
        BEGIN
            VALUE "FileDescription",  "Incremental search plugin for FAR manager\0"
            VALUE "FileVersion",      "2.1\0"
            //VALUE "InternalName",     "IncSrch\0"
            VALUE "LegalCopyright",   "Copyright (C) 1999-2019, Stanislav V. Mekhanoshin\0"
            VALUE "Original Filename","INCSRCH\0"
            VALUE "ProductName",      "Incremental Search plugin\0"
            VALUE "ProductVersion",   "2.1\0"
            VALUE "LegalTrademarks",  "FAR (File and archive manager) is copyright (C) 1996-99 Eugene Roshal\0"
        END
    END

    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x0409, 0x04B0
    END
END
